package org.apidesign.apifest08.currency;

import java.util.Currency;

/** This is the skeleton class for your API. You need to make it public, so
 * it is accessible to your client code (currently in Task1Test.java) file.
 * <p>
 * Feel free to create additional classes or rename this one, just keep all
 * the API and its implementation in this package. Do not spread it outside
 * to other packages.
 */
public interface Convertor {
    
    /**
     * Converts by taking a request and producing a response.
     * If a convertor finds it cannot perform the requested conversion,
     * it should return a non-null {@link ConversionResult} that has null {@link ConversionResult#getNetAmount()}.
     * A convertor must not convert to a different currency than the one specified in the request.
     * <p>
     * When the need comes to extend the semantics, one subclasses the ConversionRequest and/or ConversionResult classes.
     * <p>
     * This method can be called as many times as you like.
     * A {@link Convertor} shall be considered immutable.
     * This method of a single {@link Convertor} can be called from many threads concurrently.
     * @param req the conversion request; mustn't be null
     * @return the result of carrying out the conversion request; never null
     * @throws IllegalRequestSubtypeException when the particular implementation cannot handle a specific ConversionRequest type
     */
    public ConversionResult convert( final ConversionRequest req ) throws IllegalRequestSubtypeException;

    /**
     * The request for converting a monetary amout into another currency.
     * Immutable.
     */
    public class ConversionRequest {
        
        private final MonetaryAmount srcAmount;
        private final Currency tgtCurrency;

        /**
         * A request to convert srcAmount into tgtCurrency.
         * @param srcAmount the source amount; must not be null
         * @param tgtCurrency the currency we want it in afterwards; must not be null
         */
        public ConversionRequest( final MonetaryAmount srcAmount, final Currency tgtCurrency ) {
            this.srcAmount = srcAmount;
            this.tgtCurrency = tgtCurrency;
            if ( srcAmount == null ) {
                throw new NullPointerException( "The source amount" );
            }
            if ( tgtCurrency == null ) {
                throw new NullPointerException( "The target currency" );
            }
            if ( srcAmount.getCurrency().equals( tgtCurrency ) ) {
                throw new IllegalArgumentException( "Cannot request conversion from " + srcAmount.getCurrency() + " to " + tgtCurrency );
            }
        }

        /**
         * The source amount.
         */
        public MonetaryAmount getSrcAmount() {
            return srcAmount;
        }

        /**
         * The target currency.
         */
        public Currency getTgtCurrency() {
            return tgtCurrency;
        }
        
    }
    
    /**
     * The result of converting a monetary amount into another currency.
     * For now it records just the net amount one recieves from the conversion.
     * Immutable.
     * <p>
     * <b>Extension note:</b> 
     * Other items can be added further down the road, as the need for them arises.
     * These items might provide info on other aspects of the conversion,
     * such as the fee or a reason why the conversion might not be admissible.
     */
    public class ConversionResult {
        
        private final MonetaryAmount netAmount;
        
        /**
         * A new conversion result.
         * @param netAmount the amount one recieves from the conversion; 
         * null means the conversion was not admissible
         */
        public ConversionResult( final MonetaryAmount netAmount ) {
            this.netAmount = netAmount;
        }
        
        /**
         * The amount one recieves from the conversion.
         * If null, the conversion is not admissible.
         * @return the amount
         */
        public MonetaryAmount getNetAmount() {
            return netAmount;
        }
        
    }
    
}
